%Normal modes
%Assume sound speed is independent of depth. Introduce dampening by having
%a complex sound speed. 

%Boundary for R should be rigid cylinder, i.e. dR/dr = 0, r = a (a is
%radius for cylinder), and for the surface at z = L, (L = level), 

%Chapter 2.7 in marine acoustics
%[delta^2 + k(r)]*PHI(r,w) = - F(r)

%Homogenous Helmholtz equation in cylindrical coordinates s 30.

%Separation
%PHI(r,z,w) = PHIr(r,w)*PHIz(z,w)
%Vertical wave number = y(z) = w / c * sin theta(z)
%Horisontal wave number k = w/c * cos theta(z)

%Solution of depth-part (PHIz) is (given independent speed of sound)
%PHIz(z,w) = A*e^(iyz) + B*e^(-iyz), where A and B are given by boundary
%condtitions. I will set these to PHIz(L) = 0, and dPHIz/dz(0) = 0.

%The range equation however, is a bit more bitch:
% PHIr = A0*J0(k*r)+B0*Y0(k*r), boundary conditions ==>  dR/dr = 0, r=+-a.
%When r = 0, PHIr(r,w) = A0*J0(kr)
%J0 and Y0 given at page 33. 

clear all
%constants
%dim pipe
a = 0.04;
L = 1;
r0 = 1000;
c0 = 1500-5*1i;
theta = 0;
f = 1e+6;
w = 2*pi*f;
kappa = w/c0;
lambda = real(c0)/f;

%Variables depth equation
syms z
syms Z(z)
syms y

eqZ = dsolve(diff(Z,2)+(y^2)*Z(z) == 0);
C2 = 1/2;
C3 = 1/2;
neqZ = subs(eqZ);
neqZ = simplify(neqZ);
Zz = matlabFunction(neqZ);
DZ = diff(neqZ,z);
DZz = matlabFunction(DZ);
eq1 = Zz(y,L);
eq2 = DZz(y,0);

%eq2 is always == 0, so we find y_m from eq1.
dim_m = 100;
y_m = zeros(1,dim_m);
x0 = pi/2;
for i=1:dim_m
    y_m(i) = x0;
    x0 = x0 +pi;
end

%Store the eigenfunctions Em(ym)
for i=1:dim_m
    eigenfunctions_z{i} = Zz(y_m(i),z);
end

%It is now possible to calculate velocity potential under the assumptions
%shown at page 155 in MA.
%z_s == position of source. 
z_s = 0;
k_m = zeros(1,dim_m);
syms r
for i=1:dim_m 
    k_m(i) = kappa^2-y_m(i)^2;
end
temp = 0;
for i=1:dim_m
    eq = eigenfunctions_z{i};
    eq = matlabFunction(eq);
    temp = temp + eq(z_s)*eigenfunctions_z{i}*(exp(1i*k_m(i)*r)/sqrt(k_m(i)));
end
Q = 4*pi;

PHI = ((1i*Q/sqrt(8*pi*r))*exp(-1i*(pi/4)))*temp;
PHI_F = matlabFunction(PHI);
%step length
step_l = lambda/14; %14 samples each period
%variation in Z
l_z = z_s:step_l:1;
val_phi_z = [];
for z_i=l_z
    val_phi_z = [val_phi_z;PHI_F(1/kappa^2,z_i)];
end
figure(1)
plot(l_z,val_phi_z)

%Variation in R
% l_r = 0.001:1/kappa^2:a;
% val_phi_r = [];
% for r_i=l_r
%     val_phi_r = [val_phi_r;PHI_F(r_i,0)];
% end
% figure(2)
% plot(l_r,val_phi_r)

% syms r
% syms R(r)
% syms k
% DR = diff(R);
% eqR = dsolve(diff(R,2)+(1/r)*diff(R)+((k)^2)*R == 0,DR(a) == 0);
% C7 = 1;
% neqR = subs(eqR);
% neqR = simplify(neqR);
% Rr = matlabFunction(neqR);
% 
% for i=1:dim_m
%     eigenfunctions_r{i} = Rr
% end















