clear all
%============================ FINAL PARAMETERS ============================

PML_size = 100;
pipe_length = 0.2; %[m]
pipe_radius = 0.032;%[m]
transducer_freq = 1e+6;
pulse_cycles = 15;

absorbtion = false;
if absorbtion
    abs_str = 'true';
else
    abs_str = 'false';
end

w_path = 'Results\K-wave';
filename = ['simulation' '__PL=' num2str(pipe_length) '__PR=' num2str(pipe_radius)...
    '__Freq=' num2str(transducer_freq/1e6) 'MHz' '__CYCLES=' num2str(pulse_cycles) '__ABSORBTION=' abs_str '.txt'];
%==========================================================================


 

%Pipe properties
plastic_wall_thickness = 0.003;
a = pipe_radius+plastic_wall_thickness; % pipe radius [m]
L = pipe_length; % pipe length [m]
r = 0.005; %piezo radius [m]

%============================ GRID ========================================
% create the computational grid [ 1m x 0.032 m ]
Nx = 20*128; % number of grid points in the x (row) direction
Ny = 2*128; % number of grid points in the y (column) direction
dx = L/Nx; % grid point spacing in the x direction [m]
dy = a/Ny; % grid point spacing in the y direction [m]
kgrid = makeGrid(Nx, dx, Ny, dy);

%============================= MEDIUM PROPERTIES =========================

%medium.BonA = 6;
adblue_25C_soundspeed = 1630;
adblue_25C_density = 1087;

% Hetereogenous media
medium.sound_speed = adblue_25C_soundspeed*ones(Nx, Ny); % [m/s]
medium.density = adblue_25C_density*ones(Nx, Ny);     % [kg/m^3]

medium.sound_speed(Nx-1:Nx,:) = 6000;   % [m/s]
medium.density(Nx-1:Nx,:) = 7700;     % [kg/m^3]

% define the position of the plastic wall of pipe wall
wall_thickness_GP = floor(plastic_wall_thickness/dy);
wall = zeros(Nx, Ny);
wall(1:Nx/2-1,1:wall_thickness_GP) = 1;
wall(1:Nx/2-1,end-wall_thickness_GP:end) = 1;

plastic_soundspeed = 2200;
plastic_density = 1200;
medium.sound_speed(wall == 1) = plastic_soundspeed;
medium.density(wall == 1) = plastic_density;

if absorbtion
    medium.alpha_coeff = 0.75;  % [dB/(MHz^y cm)]
    medium.alpha_power = 1.5;
end

r_GP = floor(r/dy);
% create the time array
c_max = 6000;
CFL = 0.3;
dt = CFL*dx/c_max;
%Echo nr describes how many echoes will have time to propagate in the
%simulation. 
echo_nr = 1.2;
tot_sim_time = echo_nr*(2*L)/adblue_25C_soundspeed;
%tot_sim_time = 0.0005;
%[kgrid.t_array, dt] = makeTime(kgrid, medium.sound_speed,tot_sim_time);
kgrid.t_array = 0:dt:tot_sim_time;




%================================ SOURCE ==================================
%Flat source, and curved source



%Flat source
startpoint = [floor(0.02*Nx) floor(Ny/2-r_GP)];
endpoint = [floor(0.02*Nx) floor(Ny/2+r_GP)];

%============= P0 ================
% source.p0 = 10*makeLine(Nx,Ny,startpoint,endpoint);

%============ TONEBURST ==========
source.p_mask = makeLine(Nx,Ny,startpoint,endpoint);
source_strength = (5e6)/(1630*1087);          % [Pa]
tone_burst_freq = transducer_freq;        % [Hz]
tone_burst_cycles = pulse_cycles;

max_supported_freq = 1630/(2*dx);
fs = max_supported_freq;
% create the input signal using toneBurst 
signal = toneBurst(1/dt, tone_burst_freq, tone_burst_cycles);
source.p = zeros(1,length(kgrid.t_array));
source.p(1:length(signal)) = source_strength.*signal;
source.p = filterTimeSeries(kgrid, medium, source.p);

%curved source
% source.p_mask = makeCircle(Nx,Ny,r_GP+1,Ny/2,r_GP,2*pi);
% source.p_mask = source.p_mask-makeCircle(Nx,Ny,r_GP+1,Ny/2,r_GP,pi); 
% source.p0 = 10*source.p_mask;

figure(1)
plot(source.p(1:length(signal)))

%================================== SENSOR ================================

sensor.mask = source.p_mask;
sensor.directivity_angle = zeros(Nx,Ny);
sensor_length = 2/2*r_GP;
sensor.directivity_angle(source.p_mask  == 1) = 0;%(-1:1/sensor_length:1)*pi/2;

% define the directivity pattern
sensor.directivity_pattern = 'pressure';
sensor.record = {'p','I','p_final','p_max','p_rms'};
% sensor.mask = zeros(Nx,Ny);
% sensor.mask(:,Ny/2-r_GP:Ny/2+r_GP) = 1;

%=============================== SIMULATION ===============================
input_args = {'PMLInside', false,'PMLAlpha', [2 2],'DataCast','single','MeshPlot',false};
%input_args = {'PMLInside', false};
display_mask = source.p_mask;
%display_mask = zeros(Nx,Ny);
% run the simulation
sensor_data = kspaceFirstOrder2D(kgrid, medium, source, sensor, input_args{:},'DisplayMask',display_mask,'PlotPML',true);

% =========================================================================
% VISUALISATION
% =========================================================================

%Sum over transducer surface

time_steps = length(kgrid.t_array);
sensor_size = size(sensor_data.p,1);
p_trans = zeros(1,time_steps);
for t=1:time_steps
    tmp = 0;
    for e = 1:sensor_size
        tmp = tmp+sensor_data.p(e,t);
    end
    p_trans(t) = tmp;
end

figure(L*10)
plot(kgrid.t_array,p_trans)

figure(10*L+1);
sensor_data;
imagesc(sensor_data.p, [-1, 1]);
colormap(getColorMap);
ylabel('Sensor Position');
xlabel('Time Step');
colorbar;

res = [p_trans;kgrid.t_array];
dlmwrite([w_path '\' filename],res);


